@@ -4,10 +4,20 @@
 # See the NOTICE for more information.
 
 
-import ctypes
+try:
+    import ctypes
+except MemoryError:
+    # selinux execmem denial
+    # https://bugzilla.redhat.com/show_bug.cgi?id=488396
+    ctypes = None
+except ImportError:
+    # Python on Solaris compiled with Sun Studio doesn't have ctypes
+    ctypes = None
+
 import fcntl
 import os
 import pkg_resources
+import random
 import resource
 import socket
 import sys
@@ -54,7 +64,7 @@ except ImportError:
     def _setproctitle(title):
         return
 
-def load_worker_class(uri):
+def load_class(uri, default="sync", section="gunicorn.workers"):
     if uri.startswith("egg:"):
         # uses entry points
         entry_str = uri.split("egg:")[1]
@@ -62,19 +72,20 @@ def load_worker_class(uri):
             dist, name = entry_str.rsplit("#",1)
         except ValueError:
             dist = entry_str
-            name = "sync"
+            name = default
 
-        return pkg_resources.load_entry_point(dist, "gunicorn.workers", name)
+        return pkg_resources.load_entry_point(dist, section, name)
     else:
         components = uri.split('.')
         if len(components) == 1:
             try:
                 if uri.startswith("#"):
                     uri = uri[1:]
+
                 return pkg_resources.load_entry_point("gunicorn", 
-                            "gunicorn.workers", uri)
+                            section, uri)
             except ImportError: 
-                raise RuntimeError("arbiter uri invalid or not found")
+                raise RuntimeError("class uri invalid or not found")
         klass = components.pop(-1)
         mod = __import__('.'.join(components))
         for comp in components[1:]:
@@ -87,6 +98,8 @@ def set_owner_process(uid,gid):
         try:
             os.setgid(gid)
         except OverflowError:
+            if not ctypes:
+                raise
             # versions of python < 2.6.2 don't manage unsigned int for
             # groups like on osx or fedora
             os.setgid(-ctypes.c_int(-gid).value)
@@ -98,6 +111,8 @@ def chown(path, uid, gid):
     try:
         os.chown(path, uid, gid)
     except OverflowError:
+        if not ctypes:
+            raise
         os.chown(path, uid, -ctypes.c_int(-gid).value)
 
 
@@ -178,19 +193,18 @@ def writelines(sock, lines, chunked=False):
     for line in list(lines):
         write(sock, line, chunked)
 
-def write_error(sock, msg, status_int=500, 
-        reason="Internal Server Error"):
+def write_error(sock, status_int, reason, mesg):
     html = textwrap.dedent("""\
     <html>
-        <head>
-            <title>%(reason)s</title>
-        </head>
-        <body>
-            <h1>%(reason)s</h1>
-            %(msg)s
-        </body>
+      <head>
+        <title>%(reason)s</title>
+      </head>
+      <body>
+        <h1>%(reason)s</h1>
+        %(mesg)s
+      </body>
     </html>
-    """) % {"reason": reason, "msg": msg}
+    """) % {"reason": reason, "mesg": mesg}
 
     http = textwrap.dedent("""\
     HTTP/1.1 %s %s\r
@@ -206,7 +220,7 @@ def normalize_name(name):
     return  "-".join([w.lower().capitalize() for w in name.split("-")])
     
 def import_app(module):
-    parts = module.rsplit(":", 1)
+    parts = module.split(":", 1)
     if len(parts) == 1:
         module, obj = module, "application"
     else:
@@ -254,20 +268,18 @@ def is_hoppish(header):
 
 def daemonize():
     """\
-    Standard daemonization of a process. Code is basd on the
-    ActiveState recipe at:
-        http://code.activestate.com/recipes/278731/
+    Standard daemonization of a process.
+    http://www.svbug.com/documentation/comp.unix.programmer-FAQ/faq_2.html#SEC16
     """
     if not 'GUNICORN_FD' in os.environ:
-        if os.fork() == 0: 
-            os.setsid()
-            if os.fork() != 0:
-                os.umask(0) 
-            else:
-                os._exit(0)
-        else:
+        if os.fork():
+            os._exit(0)
+        os.setsid()
+
+        if os.fork():
             os._exit(0)
         
+        os.umask(0)
         maxfd = get_maxfd()
 
         # Iterate through and close all file descriptors.
@@ -280,3 +292,9 @@ def daemonize():
         os.open(REDIRECT_TO, os.O_RDWR)
         os.dup2(0, 1)
         os.dup2(0, 2)
+
+def seed():
+    try:
+        random.seed(os.urandom(64))
+    except NotImplementedError:
+        random.seed(random.random())
