@@ -5,6 +5,7 @@
 
 from __future__ import with_statement
 
+from datetime import datetime
 import errno
 import socket
 
@@ -53,6 +54,7 @@ class AsyncWorker(base.Worker):
     def handle_request(self, req, sock, addr):
         try:
             self.cfg.pre_request(self, req)
+            request_start = datetime.now()
             resp, environ = wsgi.create(req, sock, addr, self.address, self.cfg)
             self.nr += 1
             if self.alive and self.nr >= self.max_requests:
@@ -62,22 +64,20 @@ class AsyncWorker(base.Worker):
             respiter = self.wsgi(environ, resp.start_response)
             if respiter == ALREADY_HANDLED:
                 return False
-            for item in respiter:
-                resp.write(item)
-            resp.close()
-            if hasattr(respiter, "close"):
-                respiter.close()
+            try:
+                for item in respiter:
+                    resp.write(item)
+                resp.close()
+                request_time = request_start - datetime.now()
+                self.log.access(resp, environ, request_time)
+            finally:
+                if hasattr(respiter, "close"):
+                  respiter.close()
             if resp.should_close():
                 raise StopIteration()
-        except StopIteration:
-            raise
-        except Exception, e:
-            #Only send back traceback in HTTP in debug mode.
-            self.handle_error(sock, e)
-            return False
         finally:
             try:
-                self.cfg.post_request(self, req)
+                self.cfg.post_request(self, req, environ)
             except:
                 pass
         return True
